@@ -42,18 +42,9 @@
 
 static Atom is_accelerated_atom;
 static Atom is_software_rendering_atom;
+static Atom renderer_atom;
 static gboolean property_changed;
 
-static void
-exit_1_message (const char *msg) G_GNUC_NORETURN;
-
-static void
-exit_1_message (const char *msg)
-{
-  g_printerr ("%s", msg);
-  exit (1);
-}
-
 static gboolean
 on_property_notify_timeout (gpointer data)
 {
@@ -105,10 +96,12 @@ main (int argc, char **argv)
 {
         GdkDisplay *display = NULL;
         int estatus;
-        char *child_argv[] = { LIBEXECDIR "/gnome-session-check-accelerated-helper", NULL };
+        char *gl_helper_argv[] = { LIBEXECDIR "/gnome-session-check-accelerated-gl-helper", "--print-renderer", NULL };
+        char *gles_helper_argv[] = { LIBEXECDIR "/gnome-session-check-accelerated-gles-helper", "--print-renderer", NULL };
+        char *renderer_string = NULL;
         Window rootwin;
         glong is_accelerated, is_software_rendering;
-        GError *error = NULL;
+        GError *gl_error = NULL, *gles_error = NULL;
 
         /* gnome-session-check-accelerated gets run before X is started in the wayland
          * case, and it currently requires X. Until we have that working, just always
@@ -125,6 +118,7 @@ main (int argc, char **argv)
 
         is_accelerated_atom = gdk_x11_get_xatom_by_name_for_display (display, "_GNOME_SESSION_ACCELERATED");
         is_software_rendering_atom = gdk_x11_get_xatom_by_name_for_display (display, "_GNOME_IS_SOFTWARE_RENDERING");
+        renderer_atom = gdk_x11_get_xatom_by_name_for_display (display, "_GNOME_SESSION_RENDERER");
 
         {
                 Atom type;
@@ -151,6 +145,17 @@ main (int argc, char **argv)
                                 /* else fall through and do the check ourselves */
 
                         } else {
+                                gdk_x11_display_error_trap_push (display);
+                                XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), rootwin,
+                                                    renderer_atom,
+                                                    0, G_MAXLONG, False, XA_STRING, &type, &format, &nitems,
+                                                    &bytes_after, &data);
+                                gdk_x11_display_error_trap_pop_ignored (display);
+
+                                if (type == XA_STRING) {
+                                        g_print ("%s", data);
+                                }
+
                                 return (*is_accelerated_ptr == 0 ? 1 : 0);
                         }
                 }
@@ -162,6 +167,9 @@ main (int argc, char **argv)
 
         /* First indicate that a test is in progress */
         is_accelerated = ACCEL_CHECK_RUNNING;
+        is_software_rendering = FALSE;
+        estatus = 1;
+
         XChangeProperty (GDK_DISPLAY_XDISPLAY (display),
                          rootwin,
                          is_accelerated_atom,
@@ -169,25 +177,54 @@ main (int argc, char **argv)
 
         gdk_display_sync (display);
 
-        estatus = 1;
-        if (!g_spawn_sync (NULL, (char**)child_argv, NULL, 0,
-                           NULL, NULL, NULL, NULL, &estatus, &error)) {
-                is_accelerated = FALSE;
-                is_software_rendering = FALSE;
-                g_printerr ("gnome-session-check-accelerated: Failed to run helper: %s\n", error->message);
-                g_clear_error (&error);
-        } else {
+        /* First, try the GL helper */
+        if (g_spawn_sync (NULL, (char **) gl_helper_argv, NULL, 0,
+                           NULL, NULL, &renderer_string, NULL, &estatus, &gl_error)) {
                 is_accelerated = (WEXITSTATUS(estatus) == HELPER_ACCEL) || (WEXITSTATUS(estatus) == HELPER_SOFTWARE_RENDERING);
                 is_software_rendering = (WEXITSTATUS(estatus) == HELPER_SOFTWARE_RENDERING);
-                if (!is_accelerated)
-                        g_printerr ("gnome-session-check-accelerated: Helper exited with code %d\n", estatus);
+                if (is_accelerated)
+                        goto finish;
+
+                g_clear_pointer (&renderer_string, g_free);
+                g_printerr ("gnome-session-check-accelerated: GL Helper exited with code %d\n", estatus);
         }
 
+        /* Then, try the GLES helper */
+        if (g_spawn_sync (NULL, (char **) gles_helper_argv, NULL, 0,
+                           NULL, NULL, &renderer_string, NULL, &estatus, &gles_error)) {
+                is_accelerated = (WEXITSTATUS(estatus) == HELPER_ACCEL);
+                if (is_accelerated)
+                        goto finish;
+
+                g_clear_pointer (&renderer_string, g_free);
+                g_printerr ("gnome-session-check-accelerated: GLES Helper exited with code %d\n", estatus);
+        }
+
+        /* Both helpers failed; print their error messages */
+        if (gl_error != NULL) {
+                g_printerr ("gnome-session-check-accelerated: Failed to run GL helper: %s\n", gl_error->message);
+                g_clear_error (&gl_error);
+        }
+
+        if (gles_error != NULL) {
+                g_printerr ("gnome-session-check-accelerated: Failed to run GLES helper: %s\n", gles_error->message);
+                g_clear_error (&gles_error);
+        }
+
+ finish:
 	if (is_accelerated) {
 		XChangeProperty (GDK_DISPLAY_XDISPLAY (display),
 				rootwin,
 				is_accelerated_atom,
 				XA_CARDINAL, 32, PropModeReplace, (guchar *) &is_accelerated, 1);
+
+                XChangeProperty (GDK_DISPLAY_XDISPLAY (display),
+				rootwin,
+				renderer_atom,
+				XA_STRING, 8, PropModeReplace, (guchar *) renderer_string, strlen (renderer_string));
+
+                /* Print the renderer */
+                g_print ("%s", renderer_string);
 	}
 
 	if (is_software_rendering) {
